package de.uni_marburg.powersort.benchmark;

import de.uni_marburg.powersort.data.DataEnum;
import de.uni_marburg.powersort.sort.SortEnum;
import de.uni_marburg.powersort.data.ObjectSupplier;

import java.util.Arrays;

/**
 * Custom Benchmark (CBM).
 */
public class CbmBase implements Runnable {
    private final SortEnum[] sortImplementations;
    private final DataEnum[] dataEnums;
    private final int maxDataEnumStrLen;
    private final int maxSortStrLen;

    public CbmBase(SortEnum[] sortEnums, DataEnum[] dataEnums) {
        this.sortImplementations = sortEnums;
        this.dataEnums = dataEnums;

        maxDataEnumStrLen = Arrays.stream(dataEnums).map(e -> e.toString().length()).max(Integer::compareTo).orElse(0);
        maxSortStrLen = Arrays.stream(sortEnums).map(e -> e.toString().length()).max(Integer::compareTo).orElse(0);
    }

    @Override
    public void run() {
        System.out.println();

        for (DataEnum dataEnum : dataEnums) {
            ObjectSupplier objectSupplier = null;
            for (SortEnum sortImplementation : sortImplementations) {
                if (!Filter.isFiltered(dataEnum, sortImplementation)) {
                    if (objectSupplier == null) {
                        System.out.println("Creating object supplier " + dataEnum);
                        System.out.println();
                        objectSupplier = dataEnum.getObjectSupplier();
                    }

                    String dataEnumStr = dataEnum.toString();
                    dataEnumStr += " ".repeat(maxDataEnumStrLen - dataEnumStr.length());
                    String sortImplStr = sortImplementation.toString();
                    sortImplStr += " ".repeat(maxSortStrLen - sortImplStr.length());

                    CbmBench cbmBench = new CbmBench(dataEnumStr, sortImplStr, objectSupplier, sortImplementation);
                    cbmBench.run();

                    System.out.println();
                }
            }

            System.out.println();
        }
    }
}
