package de.uni_marburg.powersort.data;

import de.uni_marburg.powersort.benchmark.IntegerArray;

public class AscendingRuns extends IntegerSupplier {
    /**
     * Can be used e.g. to construct this array:
     * [0, 1, 2, 3,  -2, -1, 0, 1,  -4, -3, -2, -1]
     *
     * @param runLength >= 1
     */
    public static AscendingRuns newAscendingRuns(
            int numOfRuns,
            int runLength,
            int decreaseBetweenRuns
    ) {
        if(numOfRuns < 0 || runLength <= 0){
            throw new IllegalArgumentException();
        }

        Integer[] data = new Integer[numOfRuns * runLength];
        for (int i = 0; i < numOfRuns; i++) {
            int low = decreaseBetweenRuns * i;
            int high = low + runLength - 1;

            Integer[] run = IntegerArray.ascending(low, high);
            System.arraycopy(run, 0, data, i * runLength, run.length);
        }
        return new AscendingRuns(data);
    }

    private AscendingRuns(Integer[] readonly) {
        super(readonly);
    }
}
